import re

routereg = re.compile(r'\./\d{6}\?\d{6}l')
prefixreg = re.compile(r"'\d+_'")
orgdatareg = re.compile(r'원자료=자료\.자료\d+')
daydatareg = re.compile(r'일일자료=자료\.자료\d+')
thnamereg = re.compile(r'성명=자료\.자료\d+')
sbnamereg = re.compile(r'자료.자료\d+\[sb\]')


def regsearch(reg, org: str) -> str:
    return reg.search(org).group(0)


def extractint(org: str) -> int:
    return int("".join([x for x in org if x.isdigit()]))



from base64 import b64encode
from json import loads
from typing import List, Tuple

from requests import get
from bs4 import BeautifulSoup

from .reg import (
    routereg, prefixreg, orgdatareg,
    daydatareg, thnamereg, sbnamereg,
    regsearch, extractint
)


def trim(lis):
    while lis and not lis[-1]: 
        del lis[-1]
    return lis


URL = 'http://112.186.146.81:4082'

comci_resp = get(f'{URL}/st')
comci_resp.encoding = 'EUC-KR'

comcigan_html = BeautifulSoup(comci_resp.text, 'html.parser')
script = comcigan_html.find_all('script')[1].contents[0]

route = regsearch(routereg, script)
PREFIX = regsearch(prefixreg, script)[1:-1]

orgnum = extractint(regsearch(orgdatareg, script))
daynum = extractint(regsearch(daydatareg, script))
thnum = extractint(regsearch(thnamereg, script))
sbnum = extractint(regsearch(sbnamereg, script))

BASEURL = f'{URL}{route[1:8]}'
SEARCHURL = f'{BASEURL}{route[8:]}'


class School:
    __slots__ = ('name', 'sccode', '_timeurl', '_week_data')

    name: str
    sccode: int
    _timeurl: str
    _week_data: List[List[List[List[Tuple[str, str, str]]]]]

    def __init__(self, name: str):
        sc_search = get(SEARCHURL + '%'.join(str(name.encode('EUC-KR')).upper()[2: -1].replace('\\X', '\\').split('\\')))
        sc_search.encoding = 'UTF-8'
        sc_list = loads(sc_search.text.replace('\0', ''))['학교검색']

        if len(sc_list) == 1:
            self.name = sc_list[0][2]
            self.sccode = sc_list[0][3]
        elif len(sc_list) > 1: 
            raise ValueError('More than one school is searched by the name passed.')
        else: 
            raise NameError('No schools have been searched by the name passed.')

        self._timeurl = f'{BASEURL}?' + b64encode(f'{PREFIX}{str(self.sccode)}_0_1'.encode('UTF-8')).decode('UTF-8')
        self._week_data = [[[[('', '', '')]]]]
        self.refresh()

    def refresh(self):
        time_res = get(self._timeurl)
        time_res.encoding = 'UTF-8'
        rawtimetable = loads(time_res.text.replace('\0', ''))

        subjects: list = rawtimetable[f'자료{sbnum}']
        long_subjects: list = rawtimetable[f'긴자료{sbnum}']
        teachers: list = rawtimetable[f'자료{thnum}']

        self._week_data = [
            [
                [
                    [
                        (
                            subjects[int(str(x)[-2:])],
                            long_subjects[int(str(x)[-2:])],
                            '' if int(str(x)[:-2]) >= len(teachers) else teachers[int(str(x)[:-2])]
                        ) for x in filter(lambda x: str(x)[:-2], trim(oneday[1:]))
                    ] for oneday in oneclass[1:6]
                ] for oneclass in onegrade
            ] for onegrade in rawtimetable[f'자료{daynum}'][1:]
        ]

    def __getitem__(self, item: int) -> List: 
        return self._week_data[item - 1]

    def __repr__(self) -> str: 
        return f'School(\'{self.name}\')'

    def __str__(self) -> str: 
        return str(self._week_data)

    def __iter__(self): 
        return iter(self._week_data)



import unittest
import asyncio

from comcigan import School, AsyncSchool


class SearchTestCase(unittest.TestCase):
    def test_correct(self):
        self.assertIsInstance(School("운정고등학교"), School)

    def test_toomany(self):
        with self.assertRaises(ValueError):
            School("금촌")

    def test_noschool(self):
        with self.assertRaises(NameError):
            School("그런거 없다.")


class AsyncSearchTestCase(unittest.TestCase):
    def test_correct(self):
        sc = AsyncSchool()
        asyncio.get_event_loop().run_until_complete(sc.init("운정고등학교"))
        self.assertIsInstance(sc, AsyncSchool)

    def test_toomany(self):
        sc = AsyncSchool()
        with self.assertRaises(ValueError):
            asyncio.get_event_loop().run_until_complete(sc.init("금촌"))

    def test_noschool(self):
        sc = AsyncSchool()
        with self.assertRaises(NameError):
            asyncio.get_event_loop().run_until_complete(sc.init("그런거 없다."))


if __name__ == "__main__":
    unittest.main()



from base64 import b64encode
from json import loads
from typing import List, Tuple

from aiohttp import ClientSession
from bs4 import BeautifulSoup

from .reg import (
    routereg,
    prefixreg,
    orgdatareg,
    daydatareg,
    thnamereg,
    sbnamereg,
    regsearch,
    extractint,
)


async def AsyncRequest(url: str, encoding: str = None):
    async with ClientSession() as sess:
        async with sess.get(url) as res:
            return await res.text(encoding)


def trim(lis):
    while lis and not lis[-1]:
        del lis[-1]
    return lis


URL = "http://112.186.146.81:4082"


class CONSTANT:
    __slots__ = ("PREFIX", "orgnum", "daynum", "thnum", "sbnum", "BASEURL", "SEARCHURL")

    async def refresh(self):
        comci_resp = await AsyncRequest(f"{URL}/st")

        comcigan_html = BeautifulSoup(comci_resp, "html.parser")
        script = comcigan_html.find_all("script")[1].contents[0]

        route = regsearch(routereg, script)
        self.PREFIX = regsearch(prefixreg, script)[1:-1]

        self.orgnum = extractint(regsearch(orgdatareg, script))
        self.daynum = extractint(regsearch(daydatareg, script))
        self.thnum = extractint(regsearch(thnamereg, script))
        self.sbnum = extractint(regsearch(sbnamereg, script))

        self.BASEURL = f"{URL}{route[1:8]}"
        self.SEARCHURL = f"{self.BASEURL}{route[8:]}"


class AsyncSchool:
    __slots__ = ("name", "sccode", "_timeurl", "_week_data", "CONSTS")

    name: str
    sccode: int
    _timeurl: str
    _week_data: List[List[List[List[Tuple[str, str, str]]]]]

    async def init(self, name: str):
        CONSTS = CONSTANT()
        await CONSTS.refresh()

        PREFIX = CONSTS.PREFIX
        BASEURL = CONSTS.BASEURL
        SEARCHURL = CONSTS.SEARCHURL

        self.CONSTS = CONSTS

        sc_search = await AsyncRequest(
            SEARCHURL
            + "%".join(str(name.encode("EUC-KR")).upper()[2:-1].replace("\\X", "\\").split("\\"))
        )
        sc_list = loads(sc_search.replace("\0", ""))["학교검색"]

        if len(sc_list) == 1:
            self.name = sc_list[0][2]
            self.sccode = sc_list[0][3]
        elif len(sc_list) > 1:
            raise ValueError("More than one school is searched by the name passed.")
        else:
            raise NameError("No schools have been searched by the name passed.")

        self._timeurl = f"{BASEURL}?" + b64encode(
            f"{PREFIX}{str(self.sccode)}_0_1".encode("UTF-8")
        ).decode("UTF-8")
        self._week_data = [[[[("", "", "")]]]]
        await self.refresh()

    async def refresh(self):
        time_res = await AsyncRequest(self._timeurl)
        rawtimetable: dict = loads(time_res.replace("\0", ""))

        sbnum = self.CONSTS.sbnum
        thnum = self.CONSTS.thnum
        daynum = self.CONSTS.daynum

        subjects: list = rawtimetable[f"자료{sbnum}"]
        long_subjects: list = rawtimetable[f"긴자료{sbnum}"]
        teachers: list = rawtimetable[f"자료{thnum}"]

        self._week_data = [
            [
                [
                    [
                        (
                            subjects[int(str(x)[-2:])],
                            long_subjects[int(str(x)[-2:])],
                            "" if int(str(x)[:-2]) >= len(teachers) else teachers[int(str(x)[:-2])],
                        )
                        for x in filter(lambda x: str(x)[:-2], trim(oneday[1:]))
                    ]
                    for oneday in oneclass[1:6]
                ]
                for oneclass in onegrade
            ]
            for onegrade in rawtimetable[f"자료{daynum}"][1:]
        ]

    def __getitem__(self, item: int) -> List:
        return self._week_data[item - 1]

    def __repr__(self) -> str:
        return f"School('{self.name}')"

    def __str__(self) -> str:
        return str(self._week_data)

    def __iter__(self):
        return iter(self._week_data)



from os import environ
from setuptools import setup, find_packages

version = environ['TRAVIS_TAG'].lstrip('v') if environ['TRAVIS'] == 'true' else environ['VERSION_NUMBER']

setup(
    name = 'comcigan',
    version = version,
    license = 'MIT',
    author = 'Team IF',
    author_email = 'Coder-Iro@teamif.io',
    description = 'Unofficial Comcigan API python wrapper',
    long_description = open('README.md').read(),
    url = 'https://github.com/Team-IF/comcigan-py',
    packages = find_packages(),
    install_requires = ['requests', 'beautifulsoup4', 'aiohttp'],
    python_requires = '>=3.7',
    classifiers = [
        # 패키지에 대한 태그
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ],
)
